﻿using System;
using System.ComponentModel;

namespace GeometricObjects
{
    public abstract class GeometricObject : IComparable, INotifyPropertyChanged
    {
        // Ereignisse
        public event MovingEventHandler Moving;
        public event MovedEventHandler Moved;
        public event InvalidMeasureEventHandler InvalidMeasure;
        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        protected virtual void OnMoving(MovingEventArgs e)
        {
            if (Moving != null)
                Moving(this, e);
        }
        protected virtual void OnMoved(EventArgs e)
        {
            if (Moved != null)
                Moved(this, e);
        }
        protected virtual void OnInvalidMeasure(InvalidMeasureEventArgs e)
        {
            if (InvalidMeasure != null)
                InvalidMeasure(this, e);
        }

        // Statische Eigenschaft
        protected static int _CountGeometricObjects;
        public static int CountGeometricObjects
        {
            get { return _CountGeometricObjects; }
        }

        // Konstruktor
        protected GeometricObject()
        {
            GeometricObject._CountGeometricObjects++;
        }

        // Eigenschaften
        private double _XCoordinate;
        public virtual double XCoordinate
        {
            get { return _XCoordinate; }
            set 
            { 
                _XCoordinate = value;
                OnPropertyChanged("XCoordinate");
            }
        }

        private double _YCoordinate;
        public virtual double YCoordinate
        {
            get { return _YCoordinate; }
            set 
            { 
                _YCoordinate = value;
                OnPropertyChanged("YCoordinate");
            }
        }

        // Abstrakte Methoden
        public abstract double GetArea();
        public abstract double GetCircumference();

        // Instanzmethoden
        public virtual int CompareTo(Object @object)
        {
            GeometricObject geoObject = @object as GeometricObject;
            if (geoObject != null)
            {
                if (GetArea() < geoObject.GetArea()) return -1;
                if (GetArea() == geoObject.GetArea()) return 0;
                return 1;
            }
            throw new ArgumentException("Ungültige Objektübergabe. Es wird der Typ 'GeometricObject' erwartet.");
        }

        public virtual void Move(double dx, double dy)
        {
            // Moving-Ereignis
            MovingEventArgs e = new MovingEventArgs();
            OnMoving(e);
            if (e.Cancel == true)
                return;
            XCoordinate += dx;
            YCoordinate += dy;
            // Moved-Ereignis
            OnMoved(new EventArgs());
        }

        // Klassenmethode
        public static int Bigger(GeometricObject object1, GeometricObject object2)
        {
            if (object1.GetArea() < object2.GetArea()) return -1;
            if (object1.GetArea() == object2.GetArea()) return 0;
            return 1;
        }
    }
}
